! This file is main file of SusHi.
! 
! written by Robert Harlander, Stefan Liebler, Hendrik Mantler
! Bergische Universitaet Wuppertal
! Version 1.4.1
! 
      program SUSHI

      implicit none
      integer i,count1,ninp,noutp
      logical ppcoll,fhflagout,thdmcflagout
      real*8 gghdum(-1:5),ggherrdum(-1:5)
      double precision SigLO,errLO,Sig_bb,err_bb,Sig_bg,err_bg
      double precision delmb,delAb,alphasPDFm
      double precision wAex(2),wLHex(2),wHHex(2)

      !common variables used by various routines
      include 'commons/common-inputoutput.f'
      include 'commons/common-consts.f'
      include 'commons/common-vars.f'
      include 'commons/common-keys.f'
      include 'commons/common-quark.f'
      include 'commons/common-int.f'
      include 'commons/common-ren.f'
      include 'commons/common-readdata.f'
      include 'commons/common-citations.f'

      common /coll/ ppcoll
      common /vgasio/ ninp,noutp

      call get_command_argument(1,jfilein)
      call get_command_argument(2,jfnameact)

      if ((len_trim(jfilein).eq.0).or.(len_trim(jfnameact).eq.0)) then
       write(*,*) "SusHi must be called with two arguments:"
       write(*,*) "./SusHi in.in out.out"
       write(*,*) "The input-file in.in has to be provided"
       write(*,*) "in the SLHA-standard. Example files: SM.in, ..."
       stop
      end if

      call initsushi(0,fhflagout,thdmcflagout,delmb,.false.)

      if (verbose.ne.0) then
         write(*,105)
         write(*,106) 'Higgs mass:',Mh
         write(*,105)
      endif

      !VEGAS output
      noutp = 6

C-{{{ cuts

      !definition of cuts
      subtr = .true.
      ptcut = .false.
      rapcut = .false.

      minpt = 0.d0
      maxpt = cme*(1-z)/2.d0
      minrap = 0.d0
      maxrap = -dlog(z)/2.d0

      pt = maxptc
      y = maxrapc

      if(dist.ge.2) then
         if((maxrapc.lt.0.d0).or.(
     &     (maxrapc.ge.maxrap).and.(.not.pseudorap) )) then
            write(*,105)
            write(*,*) 'Error: y out of range. '
            write(*,*) 'Abs[y] < ', maxrap
            write(*,105)
            stop
         endif         
      endif

      if((dist.eq.1).or.(dist.eq.3)) then
         subtr = .false.
         if(norderggh.eq.0) then
            call printinfosushi(6,'At LO, the pt of the Higgs '//
     &           'is always zero.')
            call printerrorsushi(1,6
     &           ,'Therefore, no distribution can be calulated. ')
         endif
         if((maxptc.lt.0.d0).or.(maxptc.ge.maxpt)) then
            call printerrorsushi(1,6,'Error: pt out of range. ')
         endif
      endif
      if(dist.eq.3) then
         if(maxptc.gt.
     &dsqrt(mh2*(dexp(2*maxrapc)*(1/z+z)-1-dexp(4*maxrapc)))/
     &(1+dexp(2*maxrapc))) then
            write(*,105)
            write(*,*) 'Error: pt out of range. '
            write(*,*) 'for y = ', maxrapc, ', pt must satisfy'
            write(*,*) '0 < pt < ', dsqrt(mh2*(dexp(2*maxrapc)
     &*(1/z+z)-1-dexp(4*maxrapc)))/(1+dexp(2*maxrapc))
            write(*,105)
            stop
         endif
      endif

      if(ptc.eq.0) then
      !no pt-cut
         minptc = minpt
         maxptc = maxpt
      elseif(ptc.eq.1) then
      !pt > ptmin
         if((minptc.lt.0.d0).or.(minptc.ge.maxpt)) then
            write(*,105)
            write(*,*) 'Error: ptmin out of range. '
            write(*,105)
            stop
         endif
         if(minptc.gt.0.d0) then
            subtr = .false.
            ptcut = .true.
            minpt = minptc
         endif
         maxptc = maxpt
      elseif(ptc.eq.2) then
      !pt < ptmax
         if(maxptc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ptmax out of range. '
            write(*,105)
            stop
         endif
         ptcut = .true.
         if(maxptc.gt.maxpt) then
            write(*,105)
            write(*,*) 'Warning: ptmax too large. '
            write(*,*) 'maximal value for ptmax is ', maxpt, ' GeV. '
            write(*,105)
            maxptc = maxpt
         endif
         minptc = minpt
      elseif(ptc.eq.3) then
      !ptmin < pt < ptmax
         if((minptc.lt.0.d0).or.(minptc.ge.maxpt)) then
            write(*,105)
            write(*,*) 'Error: ptmin out of range. '
            write(*,105)
            stop
         endif
         if(maxptc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ptmax out of range. '
            write(*,105)
            stop
         endif
         if(maxptc.le.minptc) then
            write(*,105) 
            write(*,*) 'Error: ptmin larger then ptmax. '
            write(*,105) 
            stop
         endif
         ptcut = .true.
         if(minptc.gt.0.d0) then
            subtr = .false.
            minpt = minptc
         endif
         if(maxptc.gt.maxpt) then
            write(*,105)
            write(*,*) 'Warning: ptmax too large. '
            write(*,*) 'maximal value for ptmax is ', maxpt, ' GeV. '
            write(*,105)
            maxptc = maxpt
         endif
      else
         write(*,105)
         write(*,*) 'Error: bad value in Block DISTRIB. '
         write(*,105)
         stop
      endif

      if(rapc.eq.0) then
      !no ycut
      elseif(rapc.eq.1) then
      !Abs[y] < ymax
         if(maxrapc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ymax out of range. '
            write(*,105)
            stop
         endif            
         rapcut = .true.
         if((maxrapc.gt.maxrap).and.(.not.pseudorap)) then
            write(*,105)
            write(*,*) 'Warning: ymax too large. '
            write(*,*) 'maximal value for ymax is ', maxrap, '. '
            write(*,105)
         else
            maxrap = maxrapc
         endif
      elseif(rapc.eq.2) then
      !Abs[y] > ymin
         if(pseudorap) then
            if(minrapc.lt.0.d0) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin'
               write(*,105)
               stop
            endif
            maxrap = 1.d100
         else
            if((minrapc.lt.0.d0).or.(minrapc.ge.maxrap)) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin < ', maxrap
               write(*,105)
               stop
            endif
         endif
         rapcut = .true.
         minrap = minrapc
      elseif(rapc.eq.3) then
      !ymin < Abs[y] < ymax
         if(pseudorap) then
            if(minrapc.lt.0.d0) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin'
               write(*,105)
               stop
            endif
            maxrap = maxrapc
         else
            if((minrapc.lt.0.d0).or.(minrapc.ge.maxrap)) then
               write(*,105)
               write(*,*) 'Error: ymin out of range. '
               write(*,*) '0 < ymin < ', maxrap
               write(*,105)
               stop
            endif
            if(maxrapc.gt.maxrap) then
               write(*,105)
               write(*,*) 'Warning: ymax too large. '
               write(*,*) 'maximal value for ymax is ', maxrap, '. '
               write(*,105)
            else
               maxrap = maxrapc
            endif
         endif
         if(maxrapc.lt.0.d0) then
            write(*,105)
            write(*,*) 'Error: ymax out of range. '
            write(*,105)
            stop
         endif            
         rapcut = .true.
         minrap = minrapc
      else
         write(*,105)
         write(*,*) 'Error: bad value in Block DISTRIB. '
         write(*,105)
         stop         
      endif

      if(rapcut.and.pseudorap.and.subtr) then
         write(*,105)
         write(*,*) 'Error: Pseudorapidity cuts cannot be ',
     &'applied when pt = 0 is allowed. '
         write(*,105)
         stop
      endif

      if((dist.eq.2).and.subtr) then
         write(*,105)
         write(*,*) 'Error: (Pseudo)rapidity distributions cannot be ',
     &'calculated when pt = 0 is allowed. '
         write(*,105)
         stop
      endif

      if(((dist.eq.1).or.(dist.eq.3)).and.ptcut) then
         write(*,105)
         write(*,*) 'Warning: Cuts in pt will be ignored ',
     &'when calculating pt-distribution. '
         write(*,105)
         ptcut = .false.
         minpt = 0.d0
c         maxpt = cme*(1-z)/2.d0
      endif

      if((dist.ge.2).and.rapcut) then
         write(*,105)
         write(*,*) 'Warning: (Pseudo)rapidity cuts will be ignored ',
     &'when calculating (pseudo)rapidity-distribution. '
         write(*,105)
         rapcut = .false.
         minrap = 0.d0
         maxrap = -dlog(z)/2.d0
      endif

      pty = .false.
      juser = .false.

      if(rapcut.and.(.not.pseudorap)) then
         pty = .true.
      endif

      if(juser) then
         pty = .false.
      endif

c      pty = .false.

C-}}}
C-{{{ calculation of cross sections - calling ggh@nnlo and bbh@nnlo

      do count1=1,3
         SUbbh(count1) = 0.D0
         SUbbherr(count1) = 0.D0
         do i=-1,5
            SUggh(count1,i) = 0.D0
            SUggherr(count1,i) = 0.D0
         enddo
      end do

      if((norderggh.eq.0).or.((dist.eq.0).and.(norderbbh.ge.0))) then
         call SUinitpdf(1,SUpdfs(1),SUiset(1))
         !for bbh alpha_s(muRfacbbh*mh) is calculated internally in bbh@nnlo
      endif
      if((norderggh.ge.1).or.((dist.eq.0).and.(norderbbh.ge.1))) then
         call SUinitpdf(2,SUpdfs(2),SUiset(2))
      endif
      if((dist.eq.0).and.(ptc.eq.0).and.(rapc.eq.0)
     &     .and.((norderggh.ge.2).or.(norderbbh.ge.2))) then
         call SUinitpdf(3,SUpdfs(3),SUiset(3))
      endif

      !Differential distributions for bbh
      if ((dist.eq.0).and.((ptc.gt.0).or.(rapc.gt.0))) then
         if (norderbbh.ge.0) then

            if (verbose.ne.0) then
               write(*,105)
               write(*,*) "Performing (N)LO bbh calculation with cuts"
               write(*,105)
            endif

            citations(9) = 1

            if (ptc.gt.0) then
               write(*,*) "Applied p_t cuts:           ",minptc,maxptc
            end if
            if (rapc.gt.0) then
               write(*,*) "Applied (pseudo)rapidity cuts:",minrap,maxrap
            end if

            call SUinitpdf(1,SUpdfs(1),SUiset(1))
            call runCouplings(murfacbbh*mh,murfacbbh*mh,0)
            call bbhdiffinit(0,cme,mh,0.d0,mt
     &           ,-1,-1,0,pseudorap,minptc,maxptc,minrap,maxrap
     &           ,murfacbbh*mh,muffacbbh*mh)
            call Sigma_bbHNLO(SigLO,errLO,Sig_bb,err_bb,
     &           Sig_bg,err_bg,SUbbh(1),SUbbherr(1))

            if (norderbbh.ge.1) then
               call SUinitpdf(2,SUpdfs(2),SUiset(2))
               call runCouplings(murfacbbh*mh,murfacbbh*mh,1)
               call bbhdiffinit(1,cme,mh,0.d0,mt
     &           ,-1,-1,0,pseudorap,minptc,maxptc,minrap,maxrap
     &           ,murfacbbh*mh,muffacbbh*mh)
               call Sigma_bbHNLO(SigLO,errLO,Sig_bb,err_bb,
     &              Sig_bg,err_bg,SUbbh(2),SUbbherr(2))
            end if

            if (norderbbh.eq.2) then
               write(*,105)
               write(*,*)
     &"Warning: Differential cross sections for bb->H are"
               write(*,*)
     &"not available at NNLO. NLO results are given."
               write(*,105)
            end if
         end if
      end if

C-}}}
C-{{{ settings for NLO calculation

      call initcouplings(fhflagout,delmb,delAb)

      if (ltotinc) then

         if (verbose.ne.0) then
            write(*,105)
            write(*,*) "Calling ggh@nnlo and bbh@nnlo"
            write(*,105)
         endif

      !calling at LO
         call XSgghbbh(0,gghdum,ggherrdum,SUbbh(1))
         do i=-1,5
            SUggh(1,i) = gghdum(i)
            SUggherr(1,i) = ggherrdum(i)
         enddo

      !calling at NLO
         call XSgghbbh(1,gghdum,ggherrdum,SUbbh(2))
         do i=-1,5
            SUggh(2,i) = gghdum(i)
            SUggherr(2,i) = ggherrdum(i)
         enddo

      !calling at NNLO
         call XSgghbbh(2,gghdum,ggherrdum,SUbbh(3))
         do i=-1,5
            SUggh(3,i) = gghdum(i)
            SUggherr(3,i) = ggherrdum(i)
         enddo

      end if

      if (verbose.ne.0) then
      if (norderggh.eq.0) then
         write(*,105)
         write(*,*) "Performing LO ggh calculation"
         write(*,105)
      else if (norderggh.gt.0) then
         write(*,105)
         write(*,*) "Performing NLO ggh calculation"
         write(*,105)
      endif
      endif

C-}}}
C-{{{ weighting of bbh results
      if (dist.eq.0) then
      !weighting bbh result for MSSM
         if ((model.eq.1).or.(model.eq.2)) then
            if (delmbfh.eq.1) then
            call bbhweights(model,twohdmver,alpha,tanb,
     &           delmb,1.D0,wAex,wLHex,wHHex)
            else 
            call bbhweights(model,twohdmver,alpha,tanb,
     &           delmb,delAb,wAex,wLHex,wHHex)
            endif

            if (norderbbh.eq.0) then
               do count1=1,3
                  if (pseudo.eq.0) then
                     SUbbh(count1) = wLHex(1) * SUbbh(count1) 
                     SUbbherr(count1) = wLHex(1) * SUbbherr(count1) 
                  else if (pseudo.eq.1) then
                     SUbbh(count1) = wAex(1) * SUbbh(count1)
                     SUbbherr(count1) = wAex(1) * SUbbherr(count1)
                  else if (pseudo.eq.2) then
                     SUbbh(count1) = wHHex(1) * SUbbh(count1)  
                     SUbbherr(count1) = wHHex(1) * SUbbherr(count1)  
                  end if
               end do
            else
               do count1=1,3
                  if (pseudo.eq.0) then
                     SUbbh(count1) = wLHex(2) * SUbbh(count1) 
                     SUbbherr(count1) = wLHex(2) * SUbbherr(count1) 
                  else if (pseudo.eq.1) then
                     SUbbh(count1) = wAex(2) * SUbbh(count1) 
                     SUbbherr(count1) = wAex(2) * SUbbherr(count1) 
                  else if (pseudo.eq.2) then
                     SUbbh(count1) = wHHex(2) * SUbbh(count1) 
                     SUbbherr(count1) = wHHex(2) * SUbbherr(count1) 
                  end if
               end do
            end if

         end if
      end if
C-}}}
C-{{{ calculation of cross sections - NLO full contributions

      lfh = 2*dlog(muffactor)
      lfr = 2*dlog(muffactor/murfactor)

      if (norderggh.ge.0) then
         if (model.eq.1) then
            call XSFull(SUggh,SUggherr,ggchan,qgchan,qqchan,ggchanerr,
     &           qgchanerr,qqchanerr,xsec,xsecerr,elw,xeff,xefferr)
         endif
         
         if (model.eq.2) then
            call renormalizeSM
            call XSFull(SUggh,SUggherr,ggchan,qgchan,qqchan,ggchanerr,
     &           qgchanerr,qqchanerr,xsec,xsecerr,elw,xeff,xefferr)
         endif
         
         if (model.eq.0) then
!set couplings to SM values:
            gc = yukfac(1)
            gt = yukfac(2)
            gb = yukfac(3)
            gtp = yukfac(4)
            gbp = yukfac(5)
            call renormalizeSM
            call XSFull(SUggh,SUggherr,ggchan,qgchan,qqchan,ggchanerr,
     &           qgchanerr,qqchanerr,xsec,xsecerr,elw,xeff,xefferr)
         end if
      end if
      
C-}}}
C-{{{ showing output/writing the file-output
      call screenoutputsushi()
      if (scalespt.and.(norderggh.ge.1)) then
         call SUinitpdf(2,SUpdfs(2),SUiset(2))
         awithpt = alphasPDFm(2,muRfactor*dsqrt(mh**2+pt**2))
         awopt = alphasPDFm(2,muRfactor*mh)
      endif
      call outputsushi(fhflagout,thdmcflagout,delmb)

      write(*,104) "Results written in output-file: ",jfnameact
      call printcopysushi(0,6,verbose)

 102  format('    ',i4,'  ',e16.8,'  # ',a21)
 103  format(a)
 104  format(a,a)
 105  format('#--------------------------------------------------#')
 106  format(a12,'    ',f10.5, '   GeV')
C-}}}

      end

!end of file
